%chi_squared_test_demo
% Example of a chi-squared test. A frequency table for the number of goals
% scored per game for Liverpool football club in the Uk Premiership
% 1995-2014 is used to generate a Poisson distribution. The predicted
% frequencies, using the distribution, are compared to actual values using
% a chi-squared test.
%
% LAST UPDATED by Andy French October 2017

function chi_squared_test_demo

%Fontsize
fsize = 18;

%Frequency table - corresponds to total goals per game. The first element
%is the number of games with no goals scored by Liverpool.
games = [56 97 90 66 34 14 3 1];
goals_per_game = 0:7;

%

%Determine probabilities for numbers of goals
p = games/sum(games);

%Determine estimate for mean number of goals per game
average_goals_per_game = sum( goals_per_game.*p );

%Determine Poisson distribution based upon this estimate
p_poisson = (average_goals_per_game.^goals_per_game)...
    *exp(-average_goals_per_game)./factorial(goals_per_game);

%Determine expected frequencies of goals per game
games_expected = p_poisson*sum(games);

%Determine chi-squared value
chi_sq = sum( (( games - games_expected ).^2)./games_expected );

%Plot expected frequencies vs data
figure('name','chi-squared test demo','color',[1 1 1])
b = bar(goals_per_game,games);
set(b,'facecolor','g');
xlabel('goals per game','fontsize',fsize);
ylabel('Frequency','fontsize',fsize);
title(['Liverpool goals per game, av=',...
    num2str( average_goals_per_game,'%4.3f' )],'fontsize',fsize);
grid on;
box on;
set(gca,'fontsize',fsize);
xlim([0,max(goals_per_game)]);

%Overlay expected frequencies using Poisson distribution
hold on
plot(goals_per_game,games_expected,'r*');

%Save PNG file
print(gcf,'liverpool goals per game.png','-dpng','-r300');
clf;

%

%Degrees of freedom for chi-squared distribution. Note one less than number
%of elements in 'games' since we have used the data average to define the
%Poisson distribution.
k = length(games) - 1;

%Chi-squared distribution PDF
xmax = 3*k;
x = linspace(0,xmax,1000);
xx = linspace(0,chi_sq,1000);
pdf = ( x.^(0.5*k-1) ).*exp(-0.5*x)./( (2.^(0.5*k)).*gamma(k/2) );
ppdf = ( xx.^(0.5*k-1) ).*exp(-0.5*xx)./( (2.^(0.5*k)).*gamma(k/2) );

%Determine probability that chi-squared value is less than or equal to
%ch_sq
P = gammainc( chi_sq/2, k/2 );

%Plot chi-squared probability density function and patch a green polygon to
%indicate the integral which yields P
patch( [xx,chi_sq,0], [ppdf,0,0],'g' );
hold on;
plot(x,pdf);
xlim([0,xmax]);
xlabel('\chi^2','fontsize',fsize);
ylabel('Probability density','fontsize',fsize);
title({'\chi^2 distribution probability density function',...
    ['degrees of freedom = ',num2str(k),', P = ',num2str(P,'%5.4f')]},'fontsize',fsize);
set(gca,'fontsize',fsize);
grid on;
box on;
print(gcf,'liverpool chi-squared test.png','-dpng','-r300');
close(gcf);

%End of code