%soccer_stats_poisson_test
% Loads goal data from an Excel file Soccer Stats.xlsx which contains all
% UK Premiership games between 1995 to 2014. For each team (and overall)
% the average goals per game are computed. This enables a Poisson
% distribution to be used to predict the fraction of matches in which a
% given team scores a particular numer of goals per game. A Chi-Squared
% test is the used to assess this prediction.
%
% LAST UPDATED by Andy French October 2017

function soccer_stats_poisson_test

%Define a vector for the number of goals per game at full time (max is nine
%in the data set for any team. Note we must have enough to map out the
%statistics for total goals per game)
goals_per_game = 0:10;

%Set chi-squared threshold value such that test data is rejected if
%P( x < chi_squared test statistic ) > P_thresh;
P_thresh = 0.95;

%

%Degrees of freedom of chi-squared distribution. Note we subtract one
%because we use the mean number of goals per game (from the data) to form
%the Poisson distribution probabilities
k = length(goals_per_game) - 1;

%Chi-squared test threshold
chi_squared_thresh = invchisq( P_thresh, k );

%%

%Load Soccer Stats data
[num,txt,raw] = xlsread( 'Soccer Stats.xlsx' );
dim = size(num);

%Get a cell array of unique team names
teams = unique( raw(6:5+dim(1),3) );

%Add 'All teams' to teams
teams{length(teams)+1} = 'All teams';

%Initialize data structure
for n=1:length(teams)
    d(n).team = teams{n};
    d(n).full_time_goals = [];
    d(n).full_time_home_goals = [];
    d(n).full_time_away_goals = [];
    d(n).full_time_total_goals = [];
    d(n).home_goals_frequencies = zeros(1,length(goals_per_game));
    d(n).home_goals_per_game_average = 0;
    d(n).away_goals_frequencies = zeros(1,length(goals_per_game));
    d(n).away_goals_per_game_average = 0;
    d(n).total_goals_frequencies = zeros(1,length(goals_per_game));
    d(n).total_goals_per_game_average = 0;
end

%Step through data and add to data structure d
for n=1:dim(1)
    
    %Extract data from the 5+n th row of the Soccer Stats spreadsheet
    match_date = raw{5+n,1};
    season = raw{5+n,2};
    home_team = raw{5+n,3};
    away_team = raw{5+n,4};
    home_goals = raw{5+n,5};
    away_goals = raw{5+n,6};
    home_goals_HT = raw{5+n,7};
    away_goals_HT = raw{5+n,8};
    home_result = raw{5+n,9};
    away_result = raw{5+n,10};
    home_points = raw{5+n,11};
    away_points = raw{5+n,12};
    
    %Find index of home team in cell array teams
    i = find( strcmp( home_team, teams ) == 1 );
    
    %Find index of away team in cell array teams
    j = find( strcmp( away_team, teams ) == 1 );
    
    %Populate data struture d. Note total goals is all goals per match
    %i.e. scored by both teams. This might be used as a metric of how
    %'exciting' a match is for a given team.
    d(i).full_time_goals = [d(i).full_time_home_goals,home_goals];
    d(j).full_time_goals = [d(j).full_time_home_goals,away_goals];
    d(i).full_time_home_goals = [d(i).full_time_home_goals,home_goals];
    d(i).full_time_total_goals = [d(i).full_time_total_goals,home_goals+away_goals];
    d(j).full_time_away_goals = [d(j).full_time_away_goals,away_goals];
    d(j).full_time_total_goals = [d(j).full_time_total_goals,home_goals+away_goals];
    
    %Add data for all teams
    d(end).full_time_goals = [d(end).full_time_goals,home_goals+away_goals];
    d(end).full_time_home_goals = [d(end).full_time_home_goals,home_goals];
    d(end).full_time_away_goals = [d(end).full_time_away_goals,away_goals];
    d(end).full_time_total_goals = [d(end).full_time_total_goals,away_goals+home_goals];
end

%Step through each team and compute statistics
for n=1:length(teams)
    
    %Goal frequency tables for each team
    d(n).goals_frequencies = hist( d(n).full_time_goals,goals_per_game );
    d(n).home_goals_frequencies = hist( d(n).full_time_home_goals,goals_per_game );
    d(n).away_goals_frequencies = hist( d(n).full_time_away_goals,goals_per_game );
    d(n).total_goals_frequencies = hist( d(n).full_time_total_goals,goals_per_game );
    
    %Compute probabilities based upon data
    d(n).p_data_goals = d(n).goals_frequencies/sum( d(n).goals_frequencies );
    d(n).p_data_home_goals = d(n).home_goals_frequencies/sum( d(n).home_goals_frequencies );
    d(n).p_data_away_goals = d(n).away_goals_frequencies/sum( d(n).away_goals_frequencies );
    d(n).p_data_total_goals = d(n).total_goals_frequencies/sum( d(n).total_goals_frequencies );
    
    %Determine expected goals per game
    d(n).goals_per_game_average = sum( goals_per_game.*d(n).p_data_goals );
    d(n).home_goals_per_game_average = sum( goals_per_game.*d(n).p_data_home_goals );
    d(n).away_goals_per_game_average = sum( goals_per_game.*d(n).p_data_away_goals );
    d(n).total_goals_per_game_average = sum( goals_per_game.*d(n).p_data_total_goals );
    
    %Store Poisson probabilities
    d(n).p_goals = poisson(goals_per_game,d(n).goals_per_game_average);
    d(n).p_home_goals = poisson(goals_per_game,d(n).home_goals_per_game_average);
    d(n).p_away_goals = poisson(goals_per_game,d(n).away_goals_per_game_average);
    d(n).p_total_goals = poisson(goals_per_game,d(n).total_goals_per_game_average);
    
    %Use Poisson distribution to predict goal frequencies
    d(n).predicted_goals_frequencies = sum( d(n).goals_frequencies )*d(n).p_goals;
    d(n).predicted_home_goals_frequencies = sum( d(n).home_goals_frequencies )*d(n).p_home_goals;
    d(n).predicted_away_goals_frequencies = sum( d(n).away_goals_frequencies )*d(n).p_away_goals;
    d(n).predicted_total_goals_frequencies  = sum( d(n).total_goals_frequencies )*d(n).p_total_goals;
    
    %Apply chi-squared test i.e. determine the probability of the
    %chi-squared value being less than or equal to the measured value.
    
    %Determine chi-squared values
    d(n).chi_squared_goals = sum ( ( (d(n).goals_frequencies -...
        d(n).predicted_goals_frequencies).^2 )./d(n).predicted_goals_frequencies );
    d(n).chi_squared_home_goals = sum ( ( (d(n).home_goals_frequencies -...
        d(n).predicted_home_goals_frequencies).^2 )./d(n).predicted_home_goals_frequencies );
    d(n).chi_squared_away_goals = sum ( ( (d(n).away_goals_frequencies -...
        d(n).predicted_away_goals_frequencies).^2 )./d(n).predicted_away_goals_frequencies );
    d(n).chi_squared_total_goals = sum ( ( (d(n).total_goals_frequencies -...
        d(n).predicted_total_goals_frequencies).^2 )./d(n).predicted_total_goals_frequencies );
    
    %Determine mean and standard deviation of chi-squared distribution
    d(n).chi_squared_mean = k ;
    d(n).chi_squared_std = sqrt( 2*k );
    
    %Determine probability of chi-squared value being less than the
    %computed statistic
    d(n).P_chi_squared = chisq( d(n).chi_squared_goals, k );
    d(n).P_chi_squared_home = chisq( d(n).chi_squared_home_goals, k );
    d(n).P_chi_squared_away = chisq( d(n).chi_squared_away_goals, k );
    d(n).P_chi_squared_total = chisq( d(n).chi_squared_total_goals, k );
end

%Step through soccer data and produce league tables based on goals per game
%averages. Ignore 'All teams' in sort, but add this to the bottom of the
%table.
all_goals_per_game = zeros(length(teams)-1,1);
home_goals_per_game = zeros(length(teams)-1,1);
away_goals_per_game = zeros(length(teams)-1,1);
total_goals_per_game = zeros(length(teams)-1,1);
for n=1:length(d)-1
    all_goals_per_game(n) = d(n).goals_per_game_average;
    home_goals_per_game(n) = d(n).home_goals_per_game_average;
    away_goals_per_game(n) = d(n).away_goals_per_game_average;
    total_goals_per_game(n) = d(n).total_goals_per_game_average;
end
[all_goals_per_game,i] = sort( all_goals_per_game,'descend' );
all_goals_per_game_league = teams(i);
all_goals_per_game(end+1) = d(end).goals_per_game_average;
all_goals_per_game_league{end+1} = 'All teams'; 
[home_goals_per_game,i] = sort( home_goals_per_game,'descend' );
home_goals_per_game_league = teams(i);
home_goals_per_game(end+1) = d(end).home_goals_per_game_average;
home_goals_per_game_league{end+1} = 'All teams'; 
[away_goals_per_game,i] = sort( away_goals_per_game,'descend'  );
away_goals_per_game_league = teams(i);
away_goals_per_game(end+1) = d(end).away_goals_per_game_average;
away_goals_per_game_league{end+1} = 'All teams'; 
[total_goals_per_game,i] = sort( total_goals_per_game,'descend'  );
total_goals_per_game_league = teams(i);
total_goals_per_game(end+1) = d(end).total_goals_per_game_average;
total_goals_per_game_league{end+1} = 'All teams'; 
    
%Save data
save( 'soccer', 'd', 'teams','P_thresh','chi_squared_thresh','goals_per_game',...
    'all_goals_per_game','all_goals_per_game_league',...
    'home_goals_per_game','home_goals_per_game_league',...
    'away_goals_per_game','away_goals_per_game_league',...
    'total_goals_per_game','total_goals_per_game_league');

%Print league tables
disp(' ');
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% ')
disp('Goals per game averages league table ')
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% ')
disp(' ')
for n=1:length(teams)-1
    disp(['Rank ',num2str(n),': goals per game average = ',...
        num2str(all_goals_per_game(n),'%4.3f'),'  ',all_goals_per_game_league{n}]);
end
disp(' ');
disp(['All team goals per game average = ',num2str(all_goals_per_game(end),'%4.3f')]);
disp(' ')
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% ')
disp('Home goals per game averages league table ')
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% ')
disp(' ')
for n=1:length(teams)-1
    disp(['Rank ',num2str(n),': goals per game average = ',...
        num2str(home_goals_per_game(n),'%4.3f'),'  ',home_goals_per_game_league{n}]);
end
disp(' ');
disp(['All team home goals per game average = ',num2str(home_goals_per_game(end),'%4.3f')]);
disp(' ')
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% ')
disp('Away goals per game averages league table ')
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% ')
disp(' ')
for n=1:length(teams)-1
    disp(['Rank ',num2str(n),': goals per game average = ',...
        num2str(away_goals_per_game(n),'%4.3f'),'  ',away_goals_per_game_league{n}]);
end
disp(' ');
disp(['All team away goals per game average = ',num2str(away_goals_per_game(end),'%4.3f')]);
disp(' ');
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% ')
disp('Total goals per game averages league table ')
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% ')
disp(' ')
for n=1:length(teams)-1
    disp(['Rank ',num2str(n),': goals per game average = ',...
        num2str(total_goals_per_game(n),'%4.3f'),'  ',total_goals_per_game_league{n}]);
end
disp(' ');
disp(['All team total goals per game average = ',num2str(total_goals_per_game(end),'%4.3f')]);
disp(' ')

%%

%poisson
% Poisson distribution probabilities for integers x, given mean rate of
% occurance lambda
function p = poisson(x,lambda)
p = ( (lambda.^x)./factorial(x) )*exp(-lambda);

%%

%chisq
% Determines the probability of a chi-squared value (x) being less than or
% equal to the specified value. This is the cumulative disribution function
% of the chi-squared distribution. k is the number of 'degrees of freedom',
% which in ost scenarios is the number of terms which comprise the
% chi-squared sum x = sum ( (observed - expected)^2 / expected ).
function p = chisq( x, k )
p = gammainc( 0.5*x,0.5*k );


%%

%Inverse cumulative distribution function for the chi-squared distribution
function x = invchisq( p, k )
x = 2*gammaincinv( p, k/2 );

%End of code